@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: am_ops.c,v 1.6.2.8 2005/01/03 20:56:11 ezk Exp $
+ * File: am-utils/amd/am_ops.c
  *
  */
 
@@ -87,9 +86,6 @@ static am_ops *vops[] =
 #ifdef HAVE_AMU_FS_UNION
   &amfs_union_ops,		/* union F/S */
 #endif /* HAVE_AMU_FS_UNION */
-#ifdef HAVE_AMU_FS_INHERIT
-  &amfs_inherit_ops,		/* inheritance F/S */
-#endif /* HAVE_AMU_FS_INHERIT */
 
   /*
    * A few more native filesystems.
@@ -126,7 +122,7 @@ static am_ops *vops[] =
 #endif /* HAVE_FS_UMAPFS */
 
   /*
-   * These 5 should be last, in the order:
+   * These 4 should be last, in the order:
    *	(1) amfs_auto
    *	(2) amfs_direct
    *	(3) amfs_toplvl
@@ -188,6 +184,10 @@ ops_showfstypes(char *buf)
 
   buf[0] = '\0';
 
+#ifdef MNTTAB_TYPE_AUTOFS
+  ops_show1(buf, &l, MNTTAB_TYPE_AUTOFS);
+#endif /* MNTTAB_TYPE_AUTOFS */
+
 #ifdef MNTTAB_TYPE_CACHEFS
   ops_show1(buf, &l, MNTTAB_TYPE_CACHEFS);
 #endif /* MNTTAB_TYPE_CACHEFS */
@@ -327,15 +327,14 @@ merge_opts(const char *opts1, const char *opts2)
        tmpstr;
        tmpstr = strtok(NULL, ",")) {
     /* copy option to temp buffer */
-    strncpy(oneopt, tmpstr, 80);
-    oneopt[79] = '\0';
+    xstrlcpy(oneopt, tmpstr, 80);
     /* if option has a value such as rsize=1024, chop the value part */
     if ((eq = haseq(oneopt)))
       *eq = '\0';
     /* find reverse option of oneopt */
     revoneopt = reverse_option(oneopt);
     /* if option orits reverse exist in opts2, ignore it */
-    if (hasmntopt(&mnt2, oneopt) || hasmntopt(&mnt2, revoneopt))
+    if (amu_hasmntopt(&mnt2, oneopt) || amu_hasmntopt(&mnt2, revoneopt))
       continue;
     /* add option to returned string */
     if (newstr && newstr[0]) {
@@ -360,10 +359,22 @@ merge_opts(const char *opts1, const char *opts2)
 
 
 am_ops *
-ops_match(am_opts *fo, char *key, char *g_key, char *path, char *keym, char *map)
+ops_search(char *type)
 {
   am_ops **vp;
   am_ops *rop = 0;
+  for (vp = vops; (rop = *vp); vp++)
+    if (STREQ(rop->fs_type, type))
+      break;
+  return rop;
+}
+
+
+am_ops *
+ops_match(am_opts *fo, char *key, char *g_key, char *path, char *keym, char *map)
+{
+  am_ops *rop = 0;
+  char *link_dir;
 
   /*
    * First crack the global opts and the local opts
@@ -377,9 +388,7 @@ ops_match(am_opts *fo, char *key, char *g_key, char *path, char *keym, char *map
     /*
      * Next find the correct filesystem type
      */
-    for (vp = vops; (rop = *vp); vp++)
-      if (STREQ(rop->fs_type, fo->opt_type))
-	break;
+    rop = ops_search(fo->opt_type);
     if (!rop) {
       plog(XLOG_USER, "fs type \"%s\" not recognized", fo->opt_type);
       rop = &amfs_error_ops;
@@ -433,18 +442,34 @@ ops_match(am_opts *fo, char *key, char *g_key, char *path, char *keym, char *map
     }
   }
 
+  /*
+   * Initialize opt_mount_type to "nfs", if it's not initialized already
+   */
+  if (!fo->opt_mount_type)
+    fo->opt_mount_type = "nfs";
+
+  /* Normalize the sublink and make it absolute */
+  link_dir = fo->opt_sublink;
+  if (link_dir && link_dir[0] && link_dir[0] != '/') {
+    link_dir = str3cat((char *) 0, fo->opt_fs, "/", link_dir);
+    normalize_slash(link_dir);
+    XFREE(fo->opt_sublink);
+    fo->opt_sublink = link_dir;
+  }
+
   /*
    * Check the filesystem is happy
    */
   if (fo->fs_mtab)
     XFREE(fo->fs_mtab);
 
-  if ((fo->fs_mtab = (*rop->fs_match) (fo)))
+  fo->fs_mtab = rop->fs_match(fo);
+  if (fo->fs_mtab)
     return rop;
 
   /*
    * Return error file system
    */
-  fo->fs_mtab = (*amfs_error_ops.fs_match) (fo);
+  fo->fs_mtab = amfs_error_ops.fs_match(fo);
   return &amfs_error_ops;
 }
