@@ -367,8 +367,7 @@ void
 new_ttl(am_node *mp)
 {
   mp->am_timeo_w = 0;
-  mp->am_ttl = clocktime();
-  mp->am_fattr.na_atime.nt_seconds = mp->am_ttl;
+  mp->am_ttl = clocktime(&mp->am_fattr.na_atime);
   mp->am_ttl += mp->am_timeo;	/* sun's -tl option */
 }
 
@@ -423,8 +422,8 @@ init_map(am_node *mp, char *dir)
   mp->am_fattr = gen_fattr;
   mp->am_fattr.na_fsid = 42;
   mp->am_fattr.na_fileid = mp->am_gen;
-  mp->am_fattr.na_atime.nt_seconds = clocktime();
-  mp->am_fattr.na_atime.nt_useconds = 0;
+  clocktime(&mp->am_fattr.na_atime);
+  /* next line copies a "struct nfstime" among several fields */
   mp->am_fattr.na_mtime = mp->am_fattr.na_ctime = mp->am_fattr.na_atime;
 
   new_ttl(mp);
@@ -587,7 +586,7 @@ map_flush_srvr(fserver *fs)
     am_node *mp = exported_ap[i];
     if (mp && mp->am_mnt && mp->am_mnt->mf_server == fs) {
       plog(XLOG_INFO, "Flushed %s; dependent on %s", mp->am_path, fs->fs_host);
-      mp->am_ttl = clocktime();
+      mp->am_ttl = clocktime(NULL);
       done = 1;
     }
   }
@@ -894,11 +893,16 @@ unmount_mp(am_node *mp)
      * symlink-cache at mount time (such as Irix 5.x and 6.x). -Erez.
      *
      * Additionally, Linux currently ignores the nt_useconds field,
-     * so we must update the nt_seconds field every time.
+     * so we must update the nt_seconds field every time if clocktime(NULL)
+     * didn't return a new number of seconds.
      */
-  if (mp->am_parent)
+  if (mp->am_parent) {
+    time_t last = mp->am_parent->am_attr.ns_u.ns_attr_u.na_mtime.nt_seconds;
+    clocktime(&mp->am_parent->am_attr.ns_u.ns_attr_u.na_mtime);
     /* defensive programming... can't we assert the above condition? */
-    mp->am_parent->am_attr.ns_u.ns_attr_u.na_mtime.nt_seconds++;
+    if (last == mp->am_parent->am_attr.ns_u.ns_attr_u.na_mtime.nt_seconds)
+      mp->am_parent->am_attr.ns_u.ns_attr_u.na_mtime.nt_seconds++;
+  }
 #endif /* not MNT2_NFS_OPT_SYMTTL */
 
   if (mf->mf_refc == 1 && !FSRV_ISUP(mf->mf_server)) {
@@ -942,7 +946,7 @@ timeout_mp(opaque_t v)				/* argument not used?! */
 {
   int i;
   time_t t = NEVER;
-  time_t now = clocktime();
+  time_t now = clocktime(NULL);
   int backoff = NumChildren / 4;
 
   dlog("Timing out automount points...");
