@@ -121,10 +121,6 @@ do_select(int smask, int fds, fd_set *fdp, struct timeval *tvp)
     errno = EINTR;
   } else {
     select_intr_valid = 1;
-    /*
-     * Invalidate the current clock value
-     */
-    clock_valid = 0;
     /*
      * Allow interrupts.  If a signal
      * occurs, then it will cause a longjmp
@@ -152,9 +148,9 @@ do_select(int smask, int fds, fd_set *fdp, struct timeval *tvp)
   /*
    * Perhaps reload the cache?
    */
-  if (do_mapc_reload < clocktime()) {
+  if (do_mapc_reload < clocktime(NULL)) {
     mapc_reload();
-    do_mapc_reload = clocktime() + gopt.map_reload_interval;
+    do_mapc_reload = clocktime(NULL) + gopt.map_reload_interval;
   }
   return nsel;
 }
@@ -195,7 +191,7 @@ run_rpc(void)
   int smask = sigblock(MASKED_SIGS);
 #endif /* not HAVE_SIGACTION */
 
-  next_softclock = clocktime();
+  next_softclock = clocktime(NULL);
 
   amd_state = Run;
 
@@ -228,7 +224,7 @@ run_rpc(void)
      * If the full timeout code is not called,
      * then recompute the time delta manually.
      */
-    now = clocktime();
+    now = clocktime(NULL);
 
     if (next_softclock <= now) {
       if (amd_state == Finishing)
