@@ -180,7 +180,7 @@ autofs_mounted(am_node *mp)
     plog(XLOG_ERROR, "AUTOFS_IOC_SETTIMEOUT: %s", strerror(errno));
 
   /* tell the daemon to call us for expirations */
-  mp->am_autofs_ttl = clocktime() + gopt.am_timeo_w;
+  mp->am_autofs_ttl = clocktime(NULL) + gopt.am_timeo_w;
 }
 
 
@@ -808,7 +808,7 @@ static int autofs_timeout_mp_task(void *arg)
 void autofs_timeout_mp(am_node *mp)
 {
   autofs_fh_t *fh = mp->am_autofs_fh;
-  time_t now = clocktime();
+  time_t now = clocktime(NULL);
 
   /* update the ttl */
   mp->am_autofs_ttl = now + gopt.am_timeo_w;
