@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: checkmount_aix.c,v 1.4.2.6 2005/03/12 20:08:30 ottavio Exp $
+ * File: am-utils/conf/checkmount/checkmount_aix.c
  *
  */
 
@@ -47,12 +46,17 @@
 #endif /* HAVE_CONFIG_H */
 #include <am_defs.h>
 
-/* external definitions */
+
+/*
+ * These were missing external definitions from old AIX's headers.  They
+ * appear to be available in <sys/vmount.h> on AIX 5.3, and possibly
+ * earlier. Hence I commented this out.
+ */
+#ifndef HAVE_EXTERN_MNTCTL
+extern int mntctl(int cmd, int size, voidp buf);
+#endif /* not HAVE_EXTERN_MNTCTL */
 extern int is_same_host(char *name1, char *name2, struct in_addr addr2);
-#if 0
-#error no longer needed on AIX 5.3
-extern int mntctl(int Command, int Size, char *Buffer);
-#endif
+
 
 int
 fixmount_check_mount(char *host, struct in_addr hostaddr, char *path)
