@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: checkmount_osf.c,v 1.3.2.5 2005/01/03 20:56:12 ezk Exp $
+ * File: am-utils/conf/checkmount/checkmount_osf.c
  *
  */
 
@@ -70,14 +69,14 @@ fixmount_check_mount(char *host, struct in_addr hostaddr, char *path)
      * Apparently two forms of nfs mount syntax are
      * accepted: host:/path or /path@host
      */
-    if (delim = strchr(fslist[i].f_mntfromname, ':')) {
+    if ((delim = strchr(fslist[i].f_mntfromname, ':'))) {
       *delim = '\0';
       if ((STREQ(delim + 1, path) ||
 	   STREQ(fslist[i].f_mntonname, path)) &&
 	  is_same_host(fslist[i].f_mntfromname,
 		       host, hostaddr))
 	  found = 1;
-    } else if (delim = strchr(fslist[i].f_mntfromname, '@')) {
+    } else if ((delim = strchr(fslist[i].f_mntfromname, '@'))) {
       *delim = '\0';
       if ((STREQ(fslist[i].f_mntfromname, path) ||
 	   STREQ(fslist[i].f_mntonname, path)) &&
