@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: nfs_prot_linux.h,v 1.5.2.11 2005/01/03 20:56:13 ezk Exp $
+ * File: am-utils/conf/nfs_prot/nfs_prot_linux.h
  *
  */
 
@@ -71,11 +70,11 @@
 
 #ifndef MNTTYPE_ISO9660
 # define MNTTYPE_ISO9660 "iso9660"
-#endif /* MNTTYPE_ISO9660 */
+#endif /* not MNTTYPE_ISO9660 */
 
 #ifndef FHSIZE
 # define FHSIZE 32
-#endif
+#endif /* not FHSIZE */
 #ifndef FHSIZE3
 # define FHSIZE3 64
 #endif /* not FHSIZE3 */
@@ -84,16 +83,29 @@
 #ifndef MNTTYPE_NFS3
 # define MNTTYPE_NFS3	"nfs"
 #endif /* not MNTTYPE_NFS3 */
-
-#ifndef MOUNTVERS3
-# define MOUNTVERS3	3
-#endif /* not MOUNTVERS3 */
-
-#ifndef NFS3_FHSIZE
-# define NFS3_FHSIZE 64
-#endif /* not NFS3_FHSIZE */
 #endif /* HAVE_FS_NFS3 */
 
+/*
+ * These two force/lazy unmount flags are sometimes missing from some linux
+ * systems' headers.
+ */
+#ifdef HAVE_UMOUNT2
+# ifndef MNT_FORCE
+#  define MNT_FORCE	0x1	/* from <sys/mount.h> */
+# endif /* not MNT_FORCE */
+# ifndef MNT_DETACH
+#  define MNT_DETACH	0x2	/* from kernel <linux/fs.h> */
+# endif /* not MNT_DETACH */
+#endif /* HAVE_UMOUNT2 */
+
+/* XXX: hack until we have a better way to detect /dev/loop devices */
+#ifdef HAVE_LINUX_LOOP_H
+# define HAVE_LOOP_DEVICE
+extern char *setup_loop_device(const char *file);
+extern int delete_loop_device(const char *device);
+#endif /* HAVE_LINUX_LOOP_H */
+
+
 /*
  * MACROS:
  */
@@ -212,11 +224,8 @@ typedef statfsres	nfsstatfsres;
 typedef symlinkargs	nfssymlinkargs;
 typedef writeargs	nfswriteargs;
 
-typedef struct mntrequest mntrequest;
-typedef struct mntres mntres;
-typedef struct umntrequest umntrequest;
-typedef struct umntres umntres;
-
+/* Autofs trick */
+typedef int autofs_args_t;
 
 /*
  * We use our own definitions here, because the definitions in the
@@ -253,79 +262,6 @@ struct nfs_args {
 };
 typedef struct nfs_args nfs_args_t;
 
-struct mntrequest {
-  char *name;
-  char *map;
-  char *opts;
-  char *path;
-};
-
-struct mntres {
-  int status;
-};
-
-struct umntrequest {
-  int isdirect;
-  u_int devid;
-  u_long rdevid;
-  struct umntrequest *next;
-};
-
-struct umntres {
-  int status;
-};
-
-#ifdef HAVE_FS_NFS3
-typedef struct {
-  u_int fhandle3_len;
-  char *fhandle3_val;
-} fhandle3;
-
-enum mountstat3 {
-       MNT_OK = 0,
-       MNT3ERR_PERM = 1,
-       MNT3ERR_NOENT = 2,
-       MNT3ERR_IO = 5,
-       MNT3ERR_ACCES = 13,
-       MNT3ERR_NOTDIR = 20,
-       MNT3ERR_INVAL = 22,
-       MNT3ERR_NAMETOOLONG = 63,
-       MNT3ERR_NOTSUPP = 10004,
-       MNT3ERR_SERVERFAULT = 10006,
-};
-typedef enum mountstat3 mountstat3;
-
-struct mountres3_ok {
-       fhandle3 fhandle;
-       struct {
-               u_int auth_flavors_len;
-               int *auth_flavors_val;
-       } auth_flavors;
-};
-typedef struct mountres3_ok mountres3_ok;
-
-struct mountres3 {
-       mountstat3 fhs_status;
-       union {
-               mountres3_ok mountinfo;
-       } mountres3_u;
-};
-typedef struct mountres3 mountres3;
-
-struct nfs_fh3 {
-  u_int fh3_length;
-  union nfs_fh3_u {
-    char data[NFS3_FHSIZE];
-  } fh3_u;
-};
-typedef struct nfs_fh3 am_nfs_fh3;
-#endif /* HAVE_FS_NFS3 */
-
-extern bool_t xdr_mntrequest(XDR *, mntrequest *);
-extern bool_t xdr_mntres(XDR *, mntres *);
-extern bool_t xdr_umntrequest(XDR *, umntrequest *);
-extern bool_t xdr_umntres(XDR *, umntres *);
-
 /*
  * Missing definitions on redhat alpha linux
  */
@@ -350,6 +286,7 @@ extern bool_t xdr_umntres(XDR *, umntres *);
 /* turn off this (b/c of hlfsd) */
 #undef HAVE_RPC_AUTH_DES_H
 
+extern int linux_version_code();
 /* use a private mapper from errno's to NFS errors */
 extern int linux_nfs_error(int e);
 #define nfs_error(e)	linux_nfs_error(e)
