@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: umount_default.c,v 1.3.2.5 2005/01/03 20:56:13 ezk Exp $
+ * File: am-utils/conf/umount/umount_default.c
  *
  */
 
@@ -54,28 +53,25 @@
 
 
 int
-umount_fs(char *fs_name, const char *mnttabname)
+umount_fs(char *mntdir, const char *mnttabname, u_int unmount_flags)
 {
   mntlist *mlist, *mp, *mp_save = 0;
   int error = 0;
 
-  mp = mlist = read_mtab(fs_name, mnttabname);
+  mp = mlist = read_mtab(mntdir, mnttabname);
 
   /*
    * Search the mount table looking for
    * the correct (ie last) matching entry
    */
   while (mp) {
-    if (STREQ(mp->mnt->mnt_fsname, fs_name) ||
-	STREQ(mp->mnt->mnt_dir, fs_name))
+    if (STREQ(mp->mnt->mnt_dir, mntdir))
       mp_save = mp;
     mp = mp->mnext;
   }
 
   if (mp_save) {
-#ifdef DEBUG
     dlog("Trying unmount(%s)", mp_save->mnt->mnt_dir);
-#endif /* DEBUG */
 
 #ifdef MOUNT_TABLE_ON_FILE
     /*
@@ -86,8 +82,18 @@ umount_fs(char *fs_name, const char *mnttabname)
     unlock_mntlist();
 #endif /* MOUNT_TABLE_ON_FILE */
 
-    if (UNMOUNT_TRAP(mp_save->mnt) < 0) {
-      switch (error = errno) {
+#ifdef NEED_AUTOFS_SPACE_HACK
+    if (unmount_flags & AMU_UMOUNT_AUTOFS) {
+      char *mnt_dir_save = mp_save->mnt->mnt_dir;
+      mp_save->mnt->mnt_dir = autofs_strdup_space_hack(mnt_dir_save);
+      error = UNMOUNT_TRAP(mp_save->mnt);
+      XFREE(mp_save->mnt->mnt_dir);
+      mp_save->mnt->mnt_dir = mnt_dir_save;
+    } else
+#endif /* NEED_AUTOFS_SPACE_HACK */
+      error = UNMOUNT_TRAP(mp_save->mnt);
+    if (error < 0) {
+      switch ((error = errno)) {
       case EINVAL:
       case ENOTBLK:
 	plog(XLOG_WARNING, "unmount: %s is not mounted", mp_save->mnt->mnt_dir);
@@ -95,25 +101,40 @@ umount_fs(char *fs_name, const char *mnttabname)
 	break;
 
       case ENOENT:
+	/*
+	 * This could happen if the kernel insists on following symlinks
+	 * when we try to unmount a direct mountpoint. We need to propagate
+	 * the error up so that the top layers know it failed and don't
+	 * try to rmdir() the mountpoint or other silly things.
+	 */
 	plog(XLOG_ERROR, "mount point %s: %m", mp_save->mnt->mnt_dir);
-	error = 0;		/* Not really an error (?) */
 	break;
 
+#if defined(MNT2_GEN_OPT_FORCE) && defined(HAVE_UMOUNT2)
+      case EBUSY:
+      case EIO:
+      case ESTALE:
+	/* caller determines if forced unmounts should be used */
+	if (unmount_flags & AMU_UMOUNT_FORCE) {
+	  error = umount2_fs(mntdir, unmount_flags);
+	  if (error < 0)
+	    error = errno;
+	  else
+	    break;		/* all is OK */
+	}
+	/* fallthrough */
+#endif /* MNT2_GEN_OPT_FORCE && HAVE_UMOUNT2 */
       default:
-#ifdef DEBUG
 	dlog("%s: unmount: %m", mp_save->mnt->mnt_dir);
-#endif /* DEBUG */
 	break;
       }
     }
-#ifdef DEBUG
     dlog("Finished unmount(%s)", mp_save->mnt->mnt_dir);
-#endif /* DEBUG */
 
-#ifdef MOUNT_TABLE_ON_FILE
     if (!error) {
+#ifdef MOUNT_TABLE_ON_FILE
       free_mntlist(mlist);
-      mp = mlist = read_mtab(fs_name, mnttabname);
+      mp = mlist = read_mtab(mntdir, mnttabname);
 
       /*
        * Search the mount table looking for
@@ -121,8 +142,7 @@ umount_fs(char *fs_name, const char *mnttabname)
        */
       mp_save = 0;
       while (mp) {
-	if (STREQ(mp->mnt->mnt_fsname, fs_name) ||
-	    STREQ(mp->mnt->mnt_dir, fs_name))
+	if (STREQ(mp->mnt->mnt_dir, mntdir))
 	  mp_save = mp;
 	mp = mp->mnext;
       }
@@ -132,12 +152,12 @@ umount_fs(char *fs_name, const char *mnttabname)
 	mp_save->mnt = 0;
 	rewrite_mtab(mlist, mnttabname);
       }
-    }
 #endif /* MOUNT_TABLE_ON_FILE */
+    }
 
   } else {
 
-    plog(XLOG_ERROR, "Couldn't find how to unmount %s", fs_name);
+    plog(XLOG_ERROR, "Couldn't find how to unmount %s", mntdir);
     /*
      * Assume it is already unmounted
      */
@@ -148,3 +168,24 @@ umount_fs(char *fs_name, const char *mnttabname)
 
   return error;
 }
+
+
+#if defined(MNT2_GEN_OPT_FORCE) && defined(HAVE_UMOUNT2)
+/* force unmount, no questions asked, without touching mnttab file */
+int
+umount2_fs(const char *mntdir, u_int unmount_flags)
+{
+  int error = 0;
+  if (unmount_flags & AMU_UMOUNT_FORCE) {
+    plog(XLOG_INFO, "umount2_fs: trying unmount/forced on %s", mntdir);
+    error = umount2(mntdir, MNT2_GEN_OPT_FORCE); /* Solaris */
+    if (error < 0 && (errno == EINVAL || errno == ENOENT))
+      error = 0;		/* ignore EINVAL/ENOENT */
+    if (error < 0)
+      plog(XLOG_WARNING, "%s: unmount/force: %m", mntdir);
+    else
+      dlog("%s: unmount/force: OK", mntdir);
+  }
+  return error;
+}
+#endif /* MNT2_GEN_OPT_FORCE && HAVE_UMOUNT2 */
