@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: fixmount.c,v 1.5.2.5 2005/01/03 20:56:14 ezk Exp $
+ * File: am-utils/fixmount/fixmount.c
  *
  */
 
@@ -120,7 +119,7 @@ is_same_host(char *name1, char *name2, struct in_addr addr2)
     } else if (!(he = gethostbyname(name1))) {
       return 0;
     } else {
-      strncpy(lasthost, name1, sizeof(lasthost) - 1);
+      xstrlcpy(lasthost, name1, MAXHOSTNAMELEN);
       memcpy(&addr1, he->h_addr, sizeof(addr1));
       return (addr1.s_addr == addr2.s_addr);
     }
@@ -164,7 +163,7 @@ remove_mount(CLIENT *client, char *host, mountlist ml, int fixit)
   struct timeval tv;
   char *pathp = dir_path;
 
-  strncpy(dir_path, ml->ml_directory, sizeof(dir_path));
+  xstrlcpy(dir_path, ml->ml_directory, sizeof(dir_path));
 
   if (!fixit) {
     printf("%s: bogus mount %s:%s\n", host, ml->ml_hostname, ml->ml_directory);
@@ -326,8 +325,7 @@ main(int argc, char *argv[])
       break;
 
     case 'h':
-      strncpy(thishost, optarg, sizeof(thishost));
-      thishost[sizeof(thishost) - 1] = '\0';
+      xstrlcpy(thishost, optarg, sizeof(thishost));
       break;
 
     case '?':
@@ -367,8 +365,7 @@ main(int argc, char *argv[])
 	      inet_ntoa(thisaddr));
       exit(1);
     }
-    strncpy(thishost, he->h_name, sizeof(thishost));
-    thishost[sizeof(thishost) - 1] = '\0';
+    xstrlcpy(thishost, he->h_name, sizeof(thishost));
   } else {
     thisaddr.s_addr = INADDR_NONE;
   }
