@@ -33,9 +33,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: fsi_util.c,v 1.3.2.8 2005/03/12 02:30:08 ottavio Exp $
+ * File: am-utils/fsinfo/fsi_util.c
  *
  */
 
@@ -307,17 +306,6 @@ compute_automount_point(char *buf, host *hp, char *vn)
 }
 
 
-char *
-xcalloc(int i, int s)
-{
-  char *p = (char *) calloc(i, (unsigned) s);
-
-  if (!p)
-    fatal("Out of memory");
-  return p;
-}
-
-
 /*
  * Data constructors..
  */
@@ -463,7 +451,7 @@ set_ether_if(ether_if *ep, int k, char *v)
 
   case EF_INADDR:{
       ep->e_inaddr.s_addr = inet_addr(v);
-      if ((int) ep->e_inaddr.s_addr == - 1)
+      if ((int) ep->e_inaddr.s_addr == -1)
 	yyerror("malformed IP dotted quad: %s", v);
       XFREE(v);
     }
