@@ -37,7 +37,7 @@
  * SUCH DAMAGE.
  *
  *
- * $Id: fsinfo.c,v 1.5.2.8 2005/01/03 20:56:14 ezk Exp $
+ * File: am-utils/fsinfo/fsinfo.c
  *
  */
 
@@ -124,7 +124,7 @@ fsi_get_args(int c, char *v[])
       break;
 
     case 'h':
-      strncpy(hostname, optarg, sizeof(hostname) - 1);
+      xstrlcpy(hostname, optarg, sizeof(hostname));
       break;
 
     case 'e':
