@@ -112,8 +112,6 @@
 #define XLOG_DEFSTR	"all,nomap,nostats"	/* Default log options */
 #define XLOG_ALL	(XLOG_FATAL|XLOG_ERROR|XLOG_USER|XLOG_WARNING|XLOG_INFO|XLOG_MAP|XLOG_STATS)
 
-#define clocktime() (clock_valid ? clock_valid : time(&clock_valid))
-
 #define NO_SUBNET	"notknown"   /* default subnet name for no subnet */
 #define	NEXP_AP		(1022)			/* gdmr: was 254 */
 #define NEXP_AP_MARGIN	(128)			/* ???? not used */
@@ -267,7 +265,6 @@ extern int xlog_level_init;
 extern serv_state amd_state;	/* Should we go now */
 extern struct in_addr myipaddr;	/* (An) IP address of this host */
 extern struct opt_tab xlog_opt[];
-extern time_t clock_valid;	/* Clock needs recalculating */
 extern u_short nfs_port;	/* Our NFS service port */
 
 /*
@@ -334,7 +331,7 @@ extern u_long get_nfs_version(char *host, struct sockaddr_in *sin, u_long nfs_ve
 extern long get_server_pid(void);
 extern int xsnprintf(char *str, size_t size, const char *format, ...);
 extern void setup_sighandler(int signum, void (*handler)(int));
-
+extern time_t clocktime(nfstime *nt);
 
 #ifdef MOUNT_TABLE_ON_FILE
 extern void rewrite_mtab(mntlist *, const char *);
