@@ -36,9 +36,8 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- *      %W% (Berkeley) %G%
  *
- * $Id: util.c,v 1.3.2.5 2005/01/03 20:56:15 ezk Exp $
+ * File: am-utils/libamu/util.c
  *
  */
 
@@ -65,7 +64,7 @@ strnsave(const char *str, int len)
 
 
 /*
- * Concatenate three strings and store in buffer pointed to
+ * Concatenate three strings and store the result in the buffer pointed to
  * by p, making p large enough to hold the strings
  */
 char *
@@ -83,6 +82,24 @@ str3cat(char *p, char *s1, char *s2, char *s3)
 }
 
 
+/*
+ * Use generic strlcpy to copy a string more carefully, null-terminating it
+ * as needed.  However, if the copied string  was truncated due to lack of
+ * space, then warn us.
+ *
+ * For now, xstrlcpy returns VOID because it doesn't look like anywhere in
+ * the Amd code do we actually use the return value of strncpy/strlcpy.
+ */
+void
+xstrlcpy(char *dst, const char *src, size_t len)
+{
+  if (len == 0)
+    return;
+  if (strlcpy(dst, src, len) >= len)
+    plog(XLOG_ERROR, "xstrlcpy: string \"%s\" truncated to \"%s\"", src, dst);
+}
+
+
 /*
  * Make all the directories in the path.
  */
@@ -109,9 +126,7 @@ mkdirs(char *path, int mode)
     if (mkdir(p2, mode) < 0) {
       error_so_far = errno;
     } else {
-#ifdef DEBUG
       dlog("mkdir(%s)", p2);
-#endif /* DEBUG */
     }
     *sp = '/';
   }
@@ -119,9 +134,7 @@ mkdirs(char *path, int mode)
   if (mkdir(p2, mode) < 0) {
     error_so_far = errno;
   } else {
-#ifdef DEBUG
     dlog("mkdir(%s)", p2);
-#endif /* DEBUG */
   }
 
   XFREE(p2);
@@ -155,13 +168,12 @@ rmdirs(char *dir)
 	if (errno != ENOTEMPTY &&
 	    errno != EBUSY &&
 	    errno != EEXIST &&
+	    errno != EROFS &&
 	    errno != EINVAL)
 	  plog(XLOG_ERROR, "rmdir(%s): %m", xdp);
 	break;
       } else {
-#ifdef DEBUG
 	dlog("rmdir(%s)", xdp);
-#endif /* DEBUG */
       }
     } else {
       break;
@@ -174,3 +186,4 @@ rmdirs(char *dir)
 
   XFREE(xdp);
 }
+
